var jxl = jxl || {
get: function (idOrElement) {
if (typeof idOrElement == 'string') {
return document.getElementById(idOrElement);
}
return idOrElement;
},
setHtml: function (idOrElement,txt) {
var elem = jxl.get(idOrElement);
if (elem) {
elem.innerHTML = txt;
}
},
setText: function (idOrElement,txt) {
var elem = jxl.get(idOrElement);
if (elem) {
if (elem.hasChildNodes()) {
elem.innerHTML = "";
}
elem.appendChild(document.createTextNode(txt));
}
},
getBy: function (idOrElement,tag,filter) {
var elem = jxl.get(idOrElement) || document;
var result = [];
if (elem) {
tag = tag || "*";
var noFilter = !(typeof filter == 'function');
var nodes = elem.getElementsByTagName(tag);
for (var i = 0, len = nodes.length; i < len; i++) {
if (noFilter || filter(nodes[i])) {
result.push(nodes[i]);
}
}
}
return result;
},
cssStrToCamelCase: function (str) {
var strArr = str.split("-");
var ret = strArr[0];
for (var i = 1, len = strArr.length; i < len; i++) {
ret += strArr[i].charAt(0).toUpperCase() + strArr[i].slice(1);
}
return ret;
},
setOpacity: function (idOrElement, val) {
var elem = jxl.get(idOrElement);
if (elem) {
elem.style.opacity = val;
elem.style.filter = 'alpha(opacity=' + (val * 100).toFixed(0) + ')';
}
},
setStyleFloat: function (idOrElement, val) {
var elem = jxl.get(idOrElement);
if (elem) {
elem.style.cssFloat = val;
elem.style.styleFloat = val;
}
},
setStyle: function (idOrElement, cssName, cssValue) {
if (typeof cssName == 'string') {
switch (cssName) {
case 'opacity':
jxl.setOpacity(idOrElement, cssValue);
break;
case 'float':
jxl.setStyleFloat(idOrElement, cssValue);
break;
default:
var elem = jxl.get(idOrElement);
if (elem) {
elem.style[jxl.cssStrToCamelCase(cssName)] = cssValue;
}
break;
}
}
},
setStyles: function (idOrElement, css) {
for (var cssName in css) {
jxl.setStyle(idOrElement, cssName, css[cssName]);
}
},
getCurrentStyle: function (idOrElement,cssName) {
var elem = jxl.get(idOrElement);
var theStyle = null;
if (elem) {
if (window.getComputedStyle) {
theStyle = window.getComputedStyle(elem, null);
}
else if (document.defaultView && document.defaultView.getComputedStyle) {
theStyle = document.defaultView.getComputedStyle(elem, null);
}
else if (elem.currentStyle) {
theStyle = elem.currentStyle;
}
}
if (!theStyle) {
theStyle = elem.style;
}
if (theStyle && cssName) {
return theStyle[jxl.cssStrToCamelCase(cssName)];
}
return theStyle;
},
reduceWhiteSpace: function (str) {
str = str.replace(/^\s+|\s+$/g, "");
str = str.replace(/[\s,\xa0]+/g, " ");
return str;
},
classNameCache: {},
classNameRegExp: function (theClassName) {
if (!jxl.classNameCache[theClassName]) {
jxl.classNameCache[theClassName] = new RegExp('(?:^|\\s+)' + theClassName + '(?:\\s+|$)');
}
return jxl.classNameCache[theClassName];
},
hasClass: function (idOrElement, theClassName) {
var elem = jxl.get(idOrElement);
if (!elem) {
return false;
}
var r = jxl.classNameRegExp(theClassName);
return r.test(elem.className);
},
addClass: function (idOrElement, theClassNames) {
var elem = jxl.get(idOrElement);
if (elem) {
var theClasses = jxl.reduceWhiteSpace(theClassNames).split(' ');
for (var i = 0, len = theClasses.length; i < len; i++) {
if (!jxl.hasClass(elem, theClasses[i])) {
elem.className = [elem.className,theClasses[i]].join(' ');
}
}
elem.className = jxl.reduceWhiteSpace(elem.className);
}
},
removeClass: function (idOrElement, theClassNames) {
var elem = jxl.get(idOrElement);
if (elem) {
var theClasses = jxl.reduceWhiteSpace(theClassNames).split(' ');
var r;
for (var i = 0, len = theClasses.length; i < len; i++) {
r = jxl.classNameRegExp(theClasses[i]);
if (r.test(elem.className)) {
elem.className = elem.className.replace(r," ");
}
}
elem.className = jxl.reduceWhiteSpace(elem.className);
}
},
isQuirksMode: function () {
return (typeof document.compatMode != 'string' || document.compatMode.indexOf('CSS') == -1);
},
toNumber: function (num,defaultNum) {
var n = parseInt(num);
return isNaN(n) ? (defaultNum || 0) : n;
},
getDoc: function () {
return (jxl.isQuirksMode()) ? document.body : document.documentElement;
},
getDocSizes: function () {
var ret = {};
var doc = jxl.getDoc();
for (var i = 0; i < arguments.length; i++) {
ret[arguments[i]] = jxl.toNumber(doc[arguments[i]],0);
}
return ret;
},
addEventHandler: function (idOrElement,eventName,handlerFunction) {
var elem = jxl.get(idOrElement);
if (elem) {
if (elem.addEventListener) {
if (eventName == 'mousewheel') {
elem.addEventListener('DOMMouseScroll', handlerFunction, false);
}
elem.addEventListener(eventName, handlerFunction, false);
return true;
}
else if (elem.attachEvent) {
elem.attachEvent("on" + eventName, handlerFunction);
return true;
}
return false;
}
return false;
},
removeEventHandler: function (idOrElement,eventName,handlerFunction) {
var elem = jxl.get(idOrElement);
if (elem) {
if (elem.addEventListener) {
if (eventName == 'mousewheel') {
elem.removeEventListener('DOMMouseScroll', handlerFunction, false);
}
elem.removeEventListener(eventName, handlerFunction, false);
}
else if (elem.attachEvent) {
elem.detachEvent("on" + eventName, handlerFunction);
}
}
},
cancelEvent: function (evt) {
evt = evt || window.event;
if(evt.stopPropagation) {
evt.stopPropagation();
}
if(evt.preventDefault) {
evt.preventDefault();
}
evt.cancelBubble = true;
evt.cancel = true;
evt.returnValue = false;
return false;
}
};
